@@ -154,7 +154,7 @@ static void dict_tcp_disconnect(DICT_TCP *dict_tcp)
 static const char *dict_tcp_lookup(DICT *dict, const char *key)
 {
     DICT_TCP *dict_tcp = (DICT_TCP *) dict;
-    char   *myname = "dict_tcp_lookup";
+    const char *myname = "dict_tcp_lookup";
     int     tries;
     char   *start;
     int     last_ch;
@@ -167,7 +167,9 @@ static const char *dict_tcp_lookup(DICT *dict, const char *key)
     /*
      * Optionally fold the key.
      */
-    if (dict->fold_buf) {
+    if (dict->flags & DICT_FLAG_FOLD_MUL) {
+	if (dict->fold_buf == 0)
+	    dict->fold_buf = vstring_alloc(10);
 	vstring_strcpy(dict->fold_buf, key);
 	key = lowercase(vstring_str(dict->fold_buf));
     }
